@@ -125,9 +125,12 @@ void dsAddValueRow(const std::vector<std::wstring>& valueVector) {
 
 //' Deactivate columns
 //'
-//' Deactivate columns of a data source in order to exclude them in generation of generative data.
-//' Note that in this version only columns of type R-class numeric and R-type double can be used in generaton of generative data.
-//' All columns of other type have to be deactivated.
+//' Deactivate columns in a data source in order to exclude them from training
+//' of generative models. Note that the training function in the package
+//' supports only columns of type R-class numeric, R-type double. All columns of
+//' other type have to be deactivated. The training function in the software
+//' service for accelerated training of generative models supports columns of
+//' any type.
 //'
 //' @param columnVector Vector of column indices
 //'
@@ -161,7 +164,7 @@ void dsDeactivateColumns(const std::vector<int>& columnVector) {
 
 //' Activate columns
 //'
-//' Activate columns of a data source in order to include them in generation of generative data. By default columns are active.
+//' Activate columns in a data source in order to include them in training of generative models. By default columns are active.
 //'
 //' @param columnVector Vector of column indices
 //'
@@ -302,7 +305,7 @@ List dsGetRow(int index) {
     
         List list;
         vector<Column*> const & columnVector = dsInt::pDataSource->getColumnVector();
-        for(int j = 0; j < columnVector.size(); j++) {
+        for(int j = 0; j < (int)columnVector.size(); j++) {
         Column::COLUMN_TYPE type = columnVector[j]->getColumnType();
             if(type == Column::STRING) {
                 StringColumn* pStringColumn = dynamic_cast<StringColumn*>(columnVector[j]);
